/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "display2D.h"

Display2D::Display2D() {
}
 
void Display2D::setOrthoParameters(double _left,double _right, double _bottom, double _top) {
	left = _left;
	right = _right;
	bottom = _bottom;
	top = _top;
}

void Display2D::setViewPortParameters(int _xWindow, int _yWindow, int _width, int _height) {
	xWindow = _xWindow;
	yWindow = _yWindow;
	width = _width;
	height = _height;
}

void Display2D::addEntity2D(Entity2D *entity) {
	entity2DList.push_back(entity);
}

void Display2D::initViewport(){
	glViewport(xWindow,yWindow,width,height);	
	glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
	gluOrtho2D(left, right, bottom, top);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();	
}
 

bool Display2D::isInViewport(int x, int y) {
	return ((left <	x) && (x < right) &&
			(bottom <y) && (y<top));
}

int Display2D::getHit(int bufSize, int xMouse, int yMouse, float delX, float delY) {
	GLuint selectBuf[bufSize];
	GLint hits;
	GLint viewport[4];

	glGetIntegerv(GL_VIEWPORT, viewport);
	glSelectBuffer(bufSize, selectBuf);
	(void) glRenderMode(GL_SELECT);
	glInitNames();
	glPushName(0.);	
	
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	gluPickMatrix((GLdouble) xMouse, (GLdouble) (viewport[3] - yMouse), delX, delY, viewport);	
	gluOrtho2D(left, right, bottom, top);
	
	//displayPicking();
	//renderClickableObjects();
	
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glFlush();
	hits = glRenderMode(GL_RENDER);	
		
	return processHits(hits, selectBuf);
}

void Display2D::renderClickableObjects() {
	initViewport ();
	for(int i=0;i<entity2DList.size();i++) {
		entity2DList[i]->drawGeometry();
	}
}

void Display2D::render() {
	initViewport ();
	for(int i=0;i<entity2DList.size();i++) {
		entity2DList[i]->render();
	}	
}
