/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "shadowMap.h"

 Matrix4 ShadowMap::scaleBiasMatrix(
    0.5, 0.,   0.,   0.5,
    0.,  0.5,  0.,   0.5,
    0.,  0.,   0.5,  0.5,
    0.,  0.,   0.,   1. );
  
 
 ShadowMap::ShadowMap(int _width, int _height, int numTex):
 	width(_width), height(_height) {
	textureUnit = GL_TEXTURE0 + numTex;

	glGenTextures(1,&texId);
	glBindTexture(GL_TEXTURE_2D,texId);
	glEnable(GL_TEXTURE_2D);

	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);	
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);    
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
	
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_COMPARE_MODE,GL_COMPARE_R_TO_TEXTURE);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_COMPARE_FUNC,GL_LEQUAL);

	glTexImage2D(GL_TEXTURE_2D,0,GL_DEPTH_COMPONENT,width,height,0,GL_DEPTH_COMPONENT,GL_UNSIGNED_BYTE,NULL);


}

ShadowMap::~ShadowMap() {
     glDeleteTextures(1,&texId);		
}


void ShadowMap::setLight(LightSpot *spot) {
	light = spot;
}

void ShadowMap::beginUpdate(float near, float far) {
    glEnable(GL_DEPTH_TEST);
    glClear(GL_DEPTH_BUFFER_BIT);
	
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    float spotAperture = light->getCutoff();
    Matrix4 projLight;
    gluPerspective(spotAperture*2.,1.,near,far);
    glGetFloatv(GL_PROJECTION_MATRIX,projLight._m);
    projLight = projLight.transpose();

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    Vector3 position = light->getPosition();
    Vector3 direction = light->getDirection();
    Vector3 target = position + direction;
    //Vector3 up = direction.perpendicular();
	Vector3 up = Vector3(0.,1.,0.);
    Matrix4 viewLight;
    gluLookAt(position.x, position.y, position.z, target.x, target.y, target.z, up.x, up.y, up.z);
    glGetFloatv(GL_MODELVIEW_MATRIX, viewLight._m);
    viewLight = viewLight.transpose();
   
    sceneToLightMatrix = scaleBiasMatrix * projLight * viewLight;
	
	glEnable(GL_POLYGON_OFFSET_FILL);
	glPolygonOffset(4.f,1.1f);
	glShadeModel(GL_FLAT);
	glColorMask(0, 0, 0, 0);	
	glViewport(0, 0, width, height);
	//glCullFace(GL_FRONT);	
}

void ShadowMap::endUpdate() {
	//glCullFace(GL_BACK);
	glShadeModel(GL_SMOOTH);
	glColorMask(1, 1, 1, 1);	
    glDisable(GL_POLYGON_OFFSET_FILL);
    //glDisable(GL_DEPTH_TEST);
	//glEnable(GL_DEPTH_TEST);

	glBindTexture(GL_TEXTURE_2D, texId);
    glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0, width, height);
}

void ShadowMap::activate() {
	glClear(GL_DEPTH_BUFFER_BIT);
	
	glActiveTexture(textureUnit);
    glBindTexture(GL_TEXTURE_2D,texId);
    glEnable(GL_TEXTURE_2D);

    glEnable(GL_TEXTURE_GEN_S);
    glTexGeni(GL_S,GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
    glTexGenfv(GL_S, GL_EYE_PLANE, sceneToLightMatrix.m[0]);

    glEnable(GL_TEXTURE_GEN_T);
    glTexGeni(GL_T,GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
    glTexGenfv(GL_T, GL_EYE_PLANE, sceneToLightMatrix.m[1]);

    glEnable(GL_TEXTURE_GEN_R);
    glTexGeni(GL_R,GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
    glTexGenfv(GL_R, GL_EYE_PLANE, sceneToLightMatrix.m[2]);	
	
    glEnable(GL_TEXTURE_GEN_Q);
    glTexGeni(GL_Q,GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
    glTexGenfv(GL_Q, GL_EYE_PLANE, sceneToLightMatrix.m[3]);

	/*glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_MODE_ARB, GL_COMPARE_R_TO_TEXTURE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_FUNC_ARB, GL_LEQUAL);
    glTexParameteri(GL_TEXTURE_2D, GL_DEPTH_TEXTURE_MODE_ARB, GL_INTENSITY);*/
	
    glActiveTexture(GL_TEXTURE0);	
}

void ShadowMap::release() {
    glActiveTexture(textureUnit);
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_TEXTURE_GEN_S);
    glDisable(GL_TEXTURE_GEN_T);
    glDisable(GL_TEXTURE_GEN_Q);
    glDisable(GL_TEXTURE_GEN_R);
    glActiveTexture(GL_TEXTURE0);	
}	
	

